"""Docstring for check_pwd.py module

Summary
-------

This module aims to check password requirements for new users
Originally written by andreatsh - andreatsh@lcm.mi.infn.it

Password requirements
---------------------

A good password should contain at least 8 characters including:
* one uppercase letter
* one lowercase letter
* one digit or one special character (better if it contains both!)

**Warning**: The first and the last character do not count!

You can also check if your password is based on strings such as user name,
last name and/or user login.

"""

import re


def is_pwd_weak(
    pstring: str, s1: str | None = None, s2: str | None = None, s3: str | None = None
) -> tuple[bool, int]:
    """Function that checks if password is weak

    Parameters
    ----------
    pstring : str
        The proposed password
    s1 : str or None, optional
        Check if the proposed password contains user name
    s2 : str or None, optional
        Check if the proposed password contains user last name
    s3 : str or None, optional
        Check if the proposed password contains user login

    Returns
    -------
    bool
        True if password is weak, False if password is strong
    int
        * ``0``, password is too short
        * ``1``, password contains blanck characters
        * ``2``, password does contain user name, or last name or user login
        * ``3``, password is good
        * ``7``, password doesn't respect all parameters required, which are:
            * one uppercase letter
            * one lowercase letter
            * one digit or one special character (better if it contains both!)

    """
    if len(pstring) < 8:
        return True, 0

    pwd = pstring

    if re.search("\s", pwd):
        return True, 1

    if s1:
        regex = re.sub("[aeiou]", ".", s1, flags=re.I)
        if re.search(regex, pwd, flags=re.IGNORECASE):
            return True, 2
    if s2:
        regex = re.sub("[aeiou]", ".", s2, flags=re.I)
        if re.search(regex, pwd, flags=re.IGNORECASE):
            return True, 2
    if s3:
        regex = re.sub("[aeiou]", ".", s3, flags=re.I)
        if re.search(regex, pwd, flags=re.IGNORECASE):
            return True, 2

    pwd = pstring[1:-1]

    if (
        re.search("[a-z]", pwd)
        and re.search("[A-Z]", pwd)
        and (re.search("[0-9]", pwd) or re.search("[^A-Za-z0-9_]", pwd))
    ):
        return False, 7
    else:
        return True, 3
