"""Docstring for lcm_ldap.py module

Summary
-------

Author : stefanopilosio - stefano.pilosio@studenti.unimi.it
# Based on the original code written by andreatsh - andreatsh@lcm.mi.infn.it

Raises
------
any
    _description_
"""

import ldap
import ldap.ldapobject
import ldap.modlist as modlist
import pwd
import sys


class LCM_ldap:
    """Class to manange LCM Ldap

    Attributes
    ----------
    conn : ldap.ldapobject.LDAPObject | None
        The LDAP connection object.
    uri : str
        The URI of the LDAP server.
    dc : str
        The domain component of the LDAP server.
    secret : str
        The password used for authentication.
    """

    def __init__(self, uri: str, cn: str, dc: str, secret: str) -> None:
        """Initialize the LCM LDAP object.

        Parameters
        ----------
        uri : str
            The URI of the LDAP server.
        cn : str
            The common name (username) used for authentication.
        dc : str
            The domain component of the LDAP server.
        secret : str
            The password used for authentication.

        Raises
        ------
        ldap.INVALID_CREDENTIALS
            If the provided username or password is incorrect.
        ldap.LDAPError
            If an LDAP-related error occurs during the connection establishment.

        """
        self.conn: ldap.ldapobject.LDAPObject | None = None
        self.uri: str = uri
        self.dc: str = dc
        self.secret: str = secret
        try:
            self.conn = ldap.initialize(self.uri)
            self.conn.protocol_version = ldap.VERSION3
            self.conn.simple_bind_s(f"{cn},{self.dc}", self.secret)
        except ldap.INVALID_CREDENTIALS:
            print("Your username or password is incorrect.")
            sys.exit(-1)
        except ldap.LDAPError as e:
            if isinstance(e.message, dict) and e.message.has_key("desc"):
                print(e.message["desc"])
            else:
                print(e)
            sys.exit(-1)
        else:
            print("Connection established.")

    def __del__(self) -> None:
        self.conn.unbind_s()

    def add_user(
        self,
        name: str,
        surname: str,
        username: str,
        usersecret,
        expireDate,
        uidNo,
        badgenum,
    ) -> None:
        """Add a new user to the LDAP directory.

        Parameters
        ----------
            name : str
                The first name of the user.
            surname : str
                The last name of the user.
            username : str
                The username of the user.
            usersecret:
                The password of the user.
            expireDate:
                The expiration date of the user's account.
            uidNo:
                The UID number of the user.
            badgenum:
                The badge number of the user.

        Returns
        -------
            None: This method does not return anything.

        Raises
        ------
            None: This method does not raise any exceptions.
        """

        if self.user_exists_byuid(username):
            print("User %s already exists!" % username)
            return

        dn = f"uid={username},ou=People,{self.dc}"

        attrs: dict = {}
        attrs["uid"] = username
        attrs["userPassword"] = usersecret
        attrs["givenName"] = name
        attrs["sn"] = surname
        attrs["cn"] = f"{name} {surname}"
        attrs["objectClass"] = [
            "person",
            "organizationalPerson",
            "inetOrgPerson",
            "posixAccount",
            "top",
            "shadowAccount",
        ]
        attrs["shadowMax"] = "99999"
        attrs["shadowWarning"] = "7"
        attrs["shadowExpire"] = expireDate
        attrs["loginShell"] = "/bin/bash"
        attrs["uidNumber"] = uidNo
        attrs["gidNumber"] = "100"
        attrs["homeDirectory"] = f"/home/{username}"
        attrs["gecos"] = f"{name} {surname},,,,{badgenum}"
        attrs["employeeNumber"] = badgenum
        attrs["mail"] = f"{username}@lcm.mi.infn.it"

        # Convert our dict to nice syntax for the add-function using modlist-module
        ldif = modlist.addModlist(attrs)

        # Do the actual synchronous add-operation to the ldap uri
        self.conn.add_s(dn, ldif)

    def user_exists_byuid(self, username: str) -> bool:
        """Function that checks if a user exists in the LDAP directory using its uid.

        Parameters
        ----------
        username : str
            The username of the user.

        Returns
        -------
        bool:
            True if the user exists, False otherwise.
        """
        baseDN: str = f"ou=People,{self.dc}"
        searchScope = ldap.SCOPE_SUBTREE
        searchFilter: str = f"uid={username}"

        try:
            result = self.conn.search_s(baseDN, searchScope, searchFilter, None)

        except ldap.LDAPError as e:
            print(e)
            return False
        else:
            if result == []:
                return False
            else:
                return True

    def get_user_cred_byuid(self, username: str):
        """Retrieves the user credentials (name and surname) based on the given username.

        Parameters
        ----------
        username : str
            The username to search for.

        Returns
        -------
        list: A list containing the user's name and surname.

        Raises
        ------
        ldap.LDAPError: If an LDAP error occurs during the search.
        """

        base_DN = f"ou=People,{self.dc}"
        search_scope = ldap.SCOPE_SUBTREE
        search_filter = f"uid={username}"
        search_attr_list = ["givenName", "sn"]

        try:
            # Result is a list of tuple (one for each match) of dn and attrs.
            # Attrs is a dict, each value is a list of string.
            result = self.conn.search_s(
                base_DN, search_scope, search_filter, search_attr_list
            )
        except ldap.LDAPError as e:
            print(e)
            return
        else:
            if result == []:
                print("User %s does not exist!", username)
                return
            result = [value[0] for value in result[0][1].values()]
            # Returns 2-element list, name and surname
            return result

    def change_pwd(self, username: str, newsecret: str) -> None:
        """Change Password of a user.

        Parameters
        ----------
        username : str
            The username queried.
        newsecret : str
            The new password.

        Returns
        -------
        None: This method does not return anything.q

        Raises
        ------
        ldap.LDAPError: If an LDAP error occurs during the password change.
        """

        if not self.user_exists_byuid(username):
            print("User %s does not exist!", username)
            return

        dn = f"uid={username},ou=People,{self.dc}"
        try:
            self.conn.passwd(dn, None, newsecret)
        except ldap.LDAPError as e:
            print(f"Error: Can't change {username} password: {e.message['desc']}")

    def changes_had_ow_expire(self, username: str, shexp) -> None:
        """Changes the shadowExpire attribute of a user.

        Parameters
        ----------
        username : str
            Username queried.
        shexp : Any
            New shadow expire attribute.

        Returns
        -------
        None: This method does not return anything.
        """
        if not self.user_exists_byuid(username):
            print("User %s does not exist!", username)
            return

        dn = "uid=" + username + ",ou=People," + self.dc
        ldif = [(ldap.MOD_REPLACE, "shadowExpire", shexp)]
        try:
            self.conn.modify_s(dn, ldif)
        except ldap.LDAPError as e:
            print(f"Error: Can't change {username} shadowExpire: {e.message['desc']}")

    def del_user(self, username: str) -> None:
        """Delete a user from the LDAP directory.

        Parameters
        ----------
        username : str
            Username queried.
        """
        if not self.user_exists_byuid(username):
            print("User %s does not exist!", username)
            return

        dn = "uid=" + username + ",ou=People," + self.dc
        try:
            self.conn.delete_s(dn)
        except ldap.LDAPError as e:
            print(f"Error: Can't delete {username}: {e.message['desc']}")


def user_exists(string: str) -> bool:
    """Function that checks if a user exists in the system.

    Parameters
    ----------
    string : str
        The username of the user.

    Returns
    -------
    bool:
        True if the user exists, False otherwise.
    """
    try:
        pwd.getpwnam(string).pw_name
        return True
    except KeyError:
        return False
