import pwd
import subprocess
import time
from re import sub

from textual import on
from textual.reactive import reactive
from textual.validation import Length, Regex
from textual.widgets import Button, Input, Pretty, Static

import new_umanager.check_pwd as cp
import new_umanager.lcm_ldap as ldap
from new_umanager.constants import *


class NewUser(Static):
    first_name: str = ""
    last_name: str = ""
    username: str = ""
    email: str = ""
    badge_num: str = ""
    user_pass: str = ""
    pass_repe: str = ""

    def compose(self):
        yield Static("New User Mode", id="new-user-mode")
        yield Input(
            placeholder="Enter LDAP Password",
            password=True,
            id="ldap-password",
            validators=[],
        )
        yield Input(
            placeholder="Enter First Name",
            id="first-name",
            validators=[
                Regex(
                    r"[A-Za-z0-9-\s]*",
                    failure_description="First name can only contain letters, numbers.",
                )
            ],
        )

        yield Input(
            placeholder="Enter Last Name",
            id="last-name",
            validators=[
                Regex(
                    r"[A-Za-z0-9-\s]*",
                    failure_description="Username can only contain letters and numbers.",
                )
            ],
        )
        yield Input(
            placeholder="Enter Username",
            id="username",
            validators=[Regex(r"[a-z0-9_]+"), Length(minimum=3, maximum=33)],
        )
        yield Input(
            placeholder="Enter Email",
            id="email",
            validators=[
                Regex(r"(?:[a-zA-Z0-9._%+\-]+)@(?:[a-zA-Z0-9.\-]+\.[a-zA-Z]{2,})")
            ],
        )
        yield Input(
            placeholder="Enter Bagde Number",
            id="badge-number",
            validators=[Regex(r"[A-Za-z0-9_]*")],
            validate_on=["submitted"],
            valid_empty=True,
        )
        yield Input(
            placeholder="Enter Password",
            password=True,
            id="password",
            validators=[Length(minimum=8)],
            validate_on=["submitted"],
        )
        yield Input(
            placeholder="Re-enter Password",
            password=True,
            id="re-enter-password",
            validators=[Length(minimum=8)],
        )
        yield Pretty([])
        yield Button("Create User", id="create-user", variant="primary")

    def on_input_submitted(self, event: Input.Submitted) -> None:
        input_id = event.input.id
        match input_id:
            case "ldap-password":
                self.ldap = event.value
            case "first-name":
                self.first_name = event.value
            case "last-name":
                self.last_name = event.value
            case "username":
                self.username = event.value
            case "email":
                self.email = event.value
            case "badge-number":
                self.badge_num = event.value
            case "password":
                self.user_pass = event.value
            case "re-enter-password":
                self.pass_repe = event.value

    @on(Input.Changed)
    def show_invalid_reasons(self, event: Input.Changed) -> None:
        if not event.validation_result.is_valid:
            self.query_one(Pretty).update(event.validation_result.failure_descriptions)
        else:
            self.query_one(Pretty).update("")

    async def on_button_pressed(self, event: Button.Pressed) -> None:
        button_id = event.button.id
        if button_id == "create-user":
            await self.action_create_user()

    async def action_create_user(self) -> None:
        """Checks and create the user"""
        # First of all, check password
        # In my experience this is the most difficult step --andreatsh
        if self.user_pass == "" or self.pass_repe == "":
            nps.notify_confirm(NULL_PASSWORD, WARNING)
            self.user_pass = ""
            self.pass_repe = ""
            return
        if self.user_pass != self.pass_repe:
            nps.notify_confirm(WRONG_PASSWORD, WARNING)
            self.user_pass = ""
            self.pass_repe = ""
            return

        tp = cp.is_pwd_weak(
            self.user_pass.value,
            sub("\s", "", self.nname.value),
            sub("\s", "", self.surname.value),
            self.username.value,
        )
        if tp[0]:
            if tp[1] == 0:
                nps.notify_confirm(BAD_PASSWORD0, WARNING)
            elif tp[1] == 1:
                nps.notify_confirm(BAD_PASSWORD1, WARNING)
            elif tp[1] == 2:
                nps.notify_confirm(BAD_PASSWORD2, WARNING)
            elif tp[1] == 3:
                nps.notify_confirm(BAD_PASSWORD3, WARNING)
            self.user_pass = ""
            self.pass_repe = ""
            return

        # Check if the chosen username already exists
        if ldap.user_exists(self.username.value):
            errormsg = "Use" + self.username + USER_EXIST
            nps.notify_confirm(errormsg, WARNING, ditw=1)
            return

        try:
            db = ldap.LCM_ldap(
                "ldaps://nanos8.pcteor1.mi.infn.it/",
                "cn=Manager",
                "dc=nanos8,dc=pcteor1",
                self.ldap,
            )
        except:
            nps.notify_confirm(DB_CONN_FAIL, WARNING)
            self.ldap = None
            return

        # Add user on main server
        temp_list = []
        for i in pwd.getpwall():
            if i.pw_uid < 64000:
                temp_list.append(i.pw_uid)
        user_uid_no: str = str(max(temp_list) + 1)
        expiration_date: str = str((int(time.time()) + 3 * 86400 * 365) / 86400)
        add_user_cmd: list[str] = (
            f'useradd -u {user_uid_no} -c "{self.first_name},,,,{self.badge_num}" -d /home/{self.username} -g users -m -s /bin/bash -e {expiration_date} {self.username}'.split()
        )

        try:
            subprocess.run(add_user_cmd)
        except:
            nps.notify_confirm(WARNING, WARNING)
            return

        # Add user to LDAP database
        await db.add_user(
            self.first_name,
            self.last_name,
            self.username,
            self.user_pass,
            expiration_date,
            user_uid_no,
            self.badge_num,
        )

        # Configure user's mail
        mforwardfile = "/home/" + self.username + "/.forward"
        ofs = open(mforwardfile, "w")
        s_email_content: str = f"{self.username}\n{self.email}"
        ofs.write(s_email_content)
        await ofs.close()

        # Limit user's disk quota
        quotacmd = f"edquota -p lcm-quota -f /home {self.username}".split()
        await subprocess.run(quotacmd)

        nps.notify_confirm(NEW_USER_CREATED, WARNING)
